@@ -11,7 +11,7 @@
 import warnings
 import pyutilib.th as unittest
 
-from pyomo.contrib.pynumero import (
+from pyomo.contrib.pynumero.dependencies import (
     numpy_available, scipy_available, numpy as np
 )
 
